@@ -2,7 +2,7 @@
 
 import dnaio
 
-if __name__ == '__main__':
-    with dnaio.open('-') as f:
+if __name__ == "__main__":
+    with dnaio.open("-") as f:
         records = list(f)
     print(len(records))
