@@ -1,3 +1,5 @@
+import gzip
+import io
 import os
 import shutil
 import subprocess
@@ -12,6 +14,7 @@ from pytest import raises, mark
 
 import dnaio
 from dnaio import (
+    BamReader,
     FileFormatError,
     FastaFormatError,
     FastqFormatError,
@@ -28,6 +31,8 @@ from dnaio import (
 )
 from dnaio.writers import FileWriter
 from dnaio.readers import BinaryFileReader
+from dnaio._core import bytes_ascii_check
+
 
 TEST_DATA = Path(__file__).parent / "data"
 SIMPLE_FASTQ = str(TEST_DATA / "simple.fastq")
@@ -48,12 +53,12 @@ class TestFastaReader:
             reads = list(f)
         assert reads == simple_fasta
 
-    def test_bytesio(self):
+    def test_bytesio(self) -> None:
         fasta = BytesIO(b">first_sequence\nSEQUENCE1\n>second_sequence\nSEQUENCE2\n")
         reads = list(FastaReader(fasta))
         assert reads == simple_fasta
 
-    def test_with_comments(self):
+    def test_with_comments(self) -> None:
         fasta = BytesIO(
             dedent(
                 """
@@ -69,7 +74,7 @@ class TestFastaReader:
         reads = list(FastaReader(fasta))
         assert reads == simple_fasta
 
-    def test_wrong_format(self):
+    def test_wrong_format(self) -> None:
         fasta = BytesIO(
             dedent(
                 """# a comment
@@ -86,13 +91,13 @@ class TestFastaReader:
             list(FastaReader(fasta))
         assert info.value.line == 2
 
-    def test_fastareader_keeplinebreaks(self):
+    def test_fastareader_keeplinebreaks(self) -> None:
         with FastaReader("tests/data/simple.fasta", keep_linebreaks=True) as f:
             reads = list(f)
         assert reads[0] == simple_fasta[0]
         assert reads[1].sequence == "SEQUEN\nCE2"
 
-    def test_context_manager(self):
+    def test_context_manager(self) -> None:
         filename = "tests/data/simple.fasta"
         with open(filename, "rb") as f:
             assert not f.closed
@@ -113,24 +118,24 @@ class TestFastaReader:
 
 
 class TestFastqReader:
-    def test_fastqreader(self):
+    def test_fastqreader(self) -> None:
         with FastqReader(SIMPLE_FASTQ) as f:
             reads = list(f)
         assert reads == simple_fastq
 
     @mark.parametrize("buffer_size", [1, 2, 3, 5, 7, 10, 20])
-    def test_fastqreader_buffersize(self, buffer_size):
+    def test_fastqreader_buffersize(self, buffer_size) -> None:
         with FastqReader("tests/data/simple.fastq", buffer_size=buffer_size) as f:
             reads = list(f)
         assert reads == simple_fastq
 
-    def test_fastqreader_buffersize_too_small(self):
+    def test_fastqreader_buffersize_too_small(self) -> None:
         with raises(ValueError) as e:
             with FastqReader("tests/data/simple.fastq", buffer_size=0) as f:
                 _ = list(f)  # pragma: no cover
         assert "buffer size too small" in e.value.args[0]
 
-    def test_fastqreader_dos(self):
+    def test_fastqreader_dos(self) -> None:
         # DOS line breaks
         with open("tests/data/dos.fastq", "rb") as f:
             assert b"\r\n" in f.read()
@@ -140,13 +145,13 @@ class TestFastqReader:
             unix_reads = list(f)
         assert dos_reads == unix_reads
 
-    def test_fastq_wrongformat(self):
+    def test_fastq_wrongformat(self) -> None:
         with raises(FastqFormatError) as info:
             with FastqReader("tests/data/withplus.fastq") as f:
                 list(f)  # pragma: no cover
         assert info.value.line == 2
 
-    def test_empty_fastq(self):
+    def test_empty_fastq(self) -> None:
         with FastqReader(BytesIO(b"")) as fq:
             assert list(fq) == []
 
@@ -175,14 +180,14 @@ class TestFastqReader:
             (b"@r1\nACG\n+\nHHH\n@r2\nT\n+\n", 7),
         ],
     )
-    def test_fastq_incomplete(self, s, line):
+    def test_fastq_incomplete(self, s, line) -> None:
         fastq = BytesIO(s)
         with raises(FastqFormatError) as info:
             with FastqReader(fastq) as fq:
                 list(fq)
         assert info.value.line == line
 
-    def test_half_record_line_numbers(self):
+    def test_half_record_line_numbers(self) -> None:
         fastq = BytesIO(b"@r\nACG\n+\nHH\n")
         # Choose the buffer size such that only parts of the record fit
         # We want to ensure that the line number is reset properly
@@ -203,21 +208,21 @@ class TestFastqReader:
             (b"@r1\nACG\n+\nHHH\n@r2\nT\n+\n\n", 7),
         ],
     )
-    def test_differing_lengths(self, s, line):
+    def test_differing_lengths(self, s, line) -> None:
         fastq = BytesIO(s)
         with raises(FastqFormatError) as info:
             with FastqReader(fastq) as fq:
                 list(fq)
         assert info.value.line == line
 
-    def test_missing_final_newline(self):
+    def test_missing_final_newline(self) -> None:
         # Files with a missing final newline are currently allowed
         fastq = BytesIO(b"@r1\nA\n+\nH")
         with dnaio.open(fastq) as f:
             records = list(f)
         assert records == [SequenceRecord("r1", "A", "H")]
 
-    def test_non_ascii_in_record(self):
+    def test_non_ascii_in_record(self) -> None:
         # \xc4 -> Ä
         fastq = BytesIO(b"@r1\n\xc4\n+\nH")
         with pytest.raises(FastqFormatError) as e:
@@ -225,17 +230,19 @@ class TestFastqReader:
                 list(f)
             e.match("Non-ASCII")
 
-    def test_not_opened_as_binary(self):
+    def test_not_opened_as_binary(self) -> None:
         filename = "tests/data/simple.fastq"
         with open(filename, "rt") as f:
             with raises(ValueError):
-                list(dnaio.open(f))
+                list(dnaio.open(f))  # type: ignore
 
-    def test_context_manager(self):
+    def test_context_manager(self) -> None:
         filename = "tests/data/simple.fastq"
         with open(filename, "rb") as f:
             assert not f.closed
-            _ = list(dnaio.open(f))
+            reader = dnaio.open(f)
+            assert isinstance(reader, FastqReader)
+            _ = list(reader)
             assert not f.closed
         assert f.closed
 
@@ -246,7 +253,7 @@ class TestFastqReader:
             assert not sr._file.closed
         assert tmp_sr._file is None
 
-    def test_two_header_detection(self):
+    def test_two_header_detection(self) -> None:
         fastq = BytesIO(b"@r1\nACG\n+r1\nHHH\n@r2\nT\n+r2\n#\n")
         with FastqReader(fastq) as fq:
             assert fq.two_headers
@@ -257,7 +264,7 @@ class TestFastqReader:
             assert not fq.two_headers
             list(fq)
 
-    def test_second_header_not_equal(self):
+    def test_second_header_not_equal(self) -> None:
         fastq = BytesIO(b"@r1\nACG\n+xy\nXXX\n")
         with raises(FastqFormatError) as info:
             with FastqReader(fastq) as fq:
@@ -272,7 +279,7 @@ class TestOpen:
     def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
-    def test_sequence_reader(self):
+    def test_sequence_reader(self) -> None:
         # test the autodetection
         with dnaio.open("tests/data/simple.fastq") as f:
             reads = list(f)
@@ -298,7 +305,7 @@ class TestOpen:
         reads = list(dnaio.open(bio))
         assert reads == simple_fasta
 
-    def test_autodetect_fasta_format(self, tmpdir):
+    def test_autodetect_fasta_format(self, tmpdir) -> None:
         path = str(tmpdir.join("tmp.fasta"))
         with dnaio.open(path, mode="w") as f:
             assert isinstance(f, FastaWriter)
@@ -308,7 +315,7 @@ class TestOpen:
             records = list(f)
         assert records == simple_fasta
 
-    def test_write_qualities_to_fasta(self):
+    def test_write_qualities_to_fasta(self) -> None:
         path = os.path.join(self._tmpdir, "tmp.fasta")
         with dnaio.open(path, mode="w", qualities=True) as f:
             assert isinstance(f, FastaWriter)
@@ -317,7 +324,7 @@ class TestOpen:
         with dnaio.open(path) as f:
             assert list(f) == simple_fasta
 
-    def test_autodetect_fastq_format(self):
+    def test_autodetect_fastq_format(self) -> None:
         path = os.path.join(self._tmpdir, "tmp.fastq")
         with dnaio.open(path, mode="w") as f:
             assert isinstance(f, FastqWriter)
@@ -326,7 +333,7 @@ class TestOpen:
         with dnaio.open(path) as f:
             assert list(f) == simple_fastq
 
-    def test_autodetect_fastq_weird_name(self):
+    def test_autodetect_fastq_weird_name(self) -> None:
         path = os.path.join(self._tmpdir, "tmp.fastq.gz")
         with dnaio.open(path, mode="w") as f:
             assert isinstance(f, FastqWriter)
@@ -337,7 +344,7 @@ class TestOpen:
         with dnaio.open(weird_path) as f:
             assert list(f) == simple_fastq
 
-    def test_fastq_qualities_missing(self):
+    def test_fastq_qualities_missing(self) -> None:
         path = os.path.join(self._tmpdir, "tmp.fastq")
         with raises(ValueError):
             with dnaio.open(path, mode="w", qualities=False):
@@ -345,7 +352,7 @@ class TestOpen:
 
 
 class TestInterleavedReader:
-    def test(self):
+    def test(self) -> None:
         expected = [
             (
                 SequenceRecord(
@@ -372,14 +379,14 @@ class TestInterleavedReader:
             reads = list(f)
         assert reads == expected
 
-    def test_missing_partner(self):
+    def test_missing_partner(self) -> None:
         s = BytesIO(b"@r1\nACG\n+\nHHH\n")
         with raises(FileFormatError) as info:
             with InterleavedPairedEndReader(s) as isr:
                 list(isr)
         assert "Interleaved input file incomplete" in info.value.message
 
-    def test_incorrectly_paired(self):
+    def test_incorrectly_paired(self) -> None:
         s = BytesIO(b"@r1/1\nACG\n+\nHHH\n@wrong_name\nTTT\n+\nHHH\n")
         with raises(FileFormatError) as info:
             with InterleavedPairedEndReader(s) as isr:
@@ -395,7 +402,7 @@ class TestFastaWriter:
     def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
-    def test(self):
+    def test(self) -> None:
         with FastaWriter(self.path) as fw:
             fw.write("name", "CCATA")
             fw.write("name2", "HELLO")
@@ -403,7 +410,7 @@ class TestFastaWriter:
         with open(self.path) as t:
             assert t.read() == ">name\nCCATA\n>name2\nHELLO\n"
 
-    def test_linelength(self):
+    def test_linelength(self) -> None:
         with FastaWriter(self.path, line_length=3) as fw:
             fw.write("r1", "ACG")
             fw.write("r2", "CCAT")
@@ -413,7 +420,7 @@ class TestFastaWriter:
             d = t.read()
             assert d == ">r1\nACG\n>r2\nCCA\nT\n>r3\nTAC\nCAG\n"
 
-    def test_write_sequence_object(self):
+    def test_write_sequence_object(self) -> None:
         with FastaWriter(self.path) as fw:
             fw.write(SequenceRecord("name", "CCATA"))
             fw.write(SequenceRecord("name2", "HELLO"))
@@ -421,7 +428,7 @@ class TestFastaWriter:
         with open(self.path) as t:
             assert t.read() == ">name\nCCATA\n>name2\nHELLO\n"
 
-    def test_write_to_file_like_object(self):
+    def test_write_to_file_like_object(self) -> None:
         bio = BytesIO()
         with FastaWriter(bio) as fw:
             fw.write(SequenceRecord("name", "CCATA"))
@@ -430,7 +437,7 @@ class TestFastaWriter:
         assert not bio.closed
         assert not fw._file.closed
 
-    def test_write_zero_length_sequence_record(self):
+    def test_write_zero_length_sequence_record(self) -> None:
         bio = BytesIO()
         with FastaWriter(bio) as fw:
             fw.write(SequenceRecord("name", ""))
@@ -445,7 +452,7 @@ class TestFastqWriter:
     def teardown_method(self):
         shutil.rmtree(self._tmpdir)
 
-    def test(self):
+    def test(self) -> None:
         with FastqWriter(self.path) as fq:
             fq.writeseq("name", "CCATA", "!#!#!")
             fq.writeseq("name2", "HELLO", "&&&!&&")
@@ -453,7 +460,7 @@ class TestFastqWriter:
         with open(self.path) as t:
             assert t.read() == "@name\nCCATA\n+\n!#!#!\n@name2\nHELLO\n+\n&&&!&&\n"
 
-    def test_twoheaders(self):
+    def test_twoheaders(self) -> None:
         with FastqWriter(self.path, two_headers=True) as fq:
             fq.write(SequenceRecord("name", "CCATA", "!#!#!"))
             fq.write(SequenceRecord("name2", "HELLO", "&&&!&"))
@@ -463,7 +470,7 @@ class TestFastqWriter:
                 t.read() == "@name\nCCATA\n+name\n!#!#!\n@name2\nHELLO\n+name2\n&&&!&\n"
             )
 
-    def test_write_to_file_like_object(self):
+    def test_write_to_file_like_object(self) -> None:
         bio = BytesIO()
         with FastqWriter(bio) as fq:
             fq.writeseq("name", "CCATA", "!#!#!")
@@ -472,7 +479,7 @@ class TestFastqWriter:
 
 
 class TestInterleavedWriter:
-    def test(self):
+    def test(self) -> None:
         reads = [
             (
                 SequenceRecord("A/1 comment", "TTA", "##H"),
@@ -493,7 +500,7 @@ class TestInterleavedWriter:
 
 
 class TestPairedSequenceReader:
-    def test_read(self):
+    def test_read(self) -> None:
         s1 = BytesIO(b"@r1\nACG\n+\nHHH\n")
         s2 = BytesIO(b"@r2\nGTT\n+\n858\n")
         with TwoFilePairedEndReader(s1, s2) as psr:
@@ -504,7 +511,7 @@ class TestPairedSequenceReader:
                 ),
             ] == list(psr)
 
-    def test_record_names_match(self):
+    def test_record_names_match(self) -> None:
         match = record_names_match
         assert match("abc", "abc")
         assert match("abc def", "abc")
@@ -517,7 +524,7 @@ class TestPairedSequenceReader:
         assert match("abc\tcomments comments", "abc\tothers others")
         assert match("abc\tdef", "abc def")
 
-    def test_record_names_match_with_ignored_trailing_12(self):
+    def test_record_names_match_with_ignored_trailing_12(self) -> None:
         match = record_names_match
         assert match("abc/1", "abc/2")
         assert match("abc.1", "abc.2")
@@ -531,13 +538,13 @@ class TestPairedSequenceReader:
         assert not match("abc", "abc1")
         assert not match("abc", "abc2")
 
-    def test_record_names_match_with_ignored_trailing_123(self):
+    def test_record_names_match_with_ignored_trailing_123(self) -> None:
         match = record_names_match
         assert match("abc/1", "abc/3")
         assert match("abc.1 def", "abc.3 ghi")
         assert match("abc.3 def", "abc.1 ghi")
 
-    def test_missing_partner1(self):
+    def test_missing_partner1(self) -> None:
         s1 = BytesIO(b"")
         s2 = BytesIO(b"@r1\nACG\n+\nHHH\n")
 
@@ -546,7 +553,7 @@ class TestPairedSequenceReader:
                 list(psr)
         assert "There are more reads in file 2 than in file 1" in info.value.message
 
-    def test_missing_partner2(self):
+    def test_missing_partner2(self) -> None:
         s1 = BytesIO(b"@r1\nACG\n+\nHHH\n")
         s2 = BytesIO(b"")
 
@@ -555,7 +562,7 @@ class TestPairedSequenceReader:
                 list(psr)
         assert "There are more reads in file 1 than in file 2" in info.value.message
 
-    def test_empty_sequences_do_not_stop_iteration(self):
+    def test_empty_sequences_do_not_stop_iteration(self) -> None:
         s1 = BytesIO(b"@r1_1\nACG\n+\nHHH\n@r2_1\nACG\n+\nHHH\n@r3_2\nACG\n+\nHHH\n")
         s2 = BytesIO(b"@r1_1\nACG\n+\nHHH\n@r2_2\n\n+\n\n@r3_2\nACG\n+\nHHH\n")
         # Second sequence for s2 is empty but valid. Should not lead to a stop of iteration.
@@ -564,7 +571,7 @@ class TestPairedSequenceReader:
         print(seqs)
         assert len(seqs) == 3
 
-    def test_incorrectly_paired(self):
+    def test_incorrectly_paired(self) -> None:
         s1 = BytesIO(b"@r1/1\nACG\n+\nHHH\n")
         s2 = BytesIO(b"@wrong_name\nTTT\n+\nHHH\n")
         with raises(FileFormatError) as info:
@@ -582,7 +589,7 @@ class TestPairedSequenceReader:
         os.path.join("tests", "data", "with_comment.fasta"),
     ],
 )
-def test_read_stdin(path):
+def test_read_stdin(path) -> None:
     # Get number of records in the input file
     with dnaio.open(path) as f:
         expected = len(list(f))
@@ -597,12 +604,13 @@ def test_read_stdin(path):
             stdin=cat.stdout,
             stdout=subprocess.PIPE,
         ) as py:
+            assert cat.stdout is not None
             cat.stdout.close()
             # Check that the read_from_stdin.py script prints the correct number of records
             assert str(expected) == py.communicate()[0].decode().strip()
 
 
-def test_file_writer(tmp_path):
+def test_file_writer(tmp_path) -> None:
     path = tmp_path / "out.txt"
     fw = FileWriter(path)
     repr(fw)
@@ -614,7 +622,7 @@ def test_file_writer(tmp_path):
     assert "operation on closed file" in e.value.args[0]
 
 
-def test_binary_file_reader():
+def test_binary_file_reader() -> None:
     bfr = BinaryFileReader("tests/data/simple.fasta")
     repr(bfr)
     bfr.close()
@@ -624,19 +632,17 @@ def test_binary_file_reader():
     assert "operation on closed" in e.value.args[0]
 
 
-def test_fasta_writer_repr(tmp_path):
+def test_fasta_writer_repr(tmp_path) -> None:
     with FastaWriter(tmp_path / "out.fasta") as fw:
         repr(fw)
 
 
-def test_fastq_writer_repr(tmp_path):
+def test_fastq_writer_repr(tmp_path) -> None:
     with FastqWriter(tmp_path / "out.fastq") as fw:
         repr(fw)
 
 
 class TestAsciiCheck:
-    from dnaio._core import bytes_ascii_check
-
     ASCII_STRING = (
         "In het Nederlands komen bijzondere leestekens niet vaak voor.".encode("ascii")
     )
@@ -645,25 +651,25 @@ class TestAsciiCheck:
         "In späterer Zeit trat Umlaut sehr häufig analogisch ein.".encode("latin-1")
     )
 
-    def test_ascii(self):
-        assert self.bytes_ascii_check(self.ASCII_STRING)
+    def test_ascii(self) -> None:
+        assert bytes_ascii_check(self.ASCII_STRING)
 
-    def test_ascii_all_chars(self):
-        assert self.bytes_ascii_check(bytes(range(128)))
-        assert not self.bytes_ascii_check(bytes(range(129)))
+    def test_ascii_all_chars(self) -> None:
+        assert bytes_ascii_check(bytes(range(128)))
+        assert not bytes_ascii_check(bytes(range(129)))
 
-    def test_non_ascii(self):
-        assert not self.bytes_ascii_check(self.NON_ASCII_STRING)
+    def test_non_ascii(self) -> None:
+        assert not bytes_ascii_check(self.NON_ASCII_STRING)
 
-    def test_non_ascii_lengths(self):
+    def test_non_ascii_lengths(self) -> None:
         # Make sure that the function finds the non-ascii byte correctly for
         # all lengths.
         non_ascii_char = "é".encode("latin-1")
         for i in range(len(self.ASCII_STRING)):
             test_string = self.ASCII_STRING[:i] + non_ascii_char
-            assert not self.bytes_ascii_check(test_string)
+            assert not bytes_ascii_check(test_string)
 
-    def test_ascii_lengths(self):
+    def test_ascii_lengths(self) -> None:
         # Make sure the ascii check is correct even though there are non-ASCII
         # bytes directly behind the search space.
         # This ensures there is no overshoot where the algorithm checks bytes
@@ -671,11 +677,11 @@ class TestAsciiCheck:
         non_ascii_char = "é".encode("latin-1")
         for i in range(1, len(self.ASCII_STRING) + 1):
             test_string = self.ASCII_STRING[:i] + (non_ascii_char * 8)
-            assert self.bytes_ascii_check(test_string, i - 1)
+            assert bytes_ascii_check(test_string, i - 1)
 
 
 class TestRecordsAreMates:
-    def test_records_are_mates(self):
+    def test_records_are_mates(self) -> None:
         assert records_are_mates(
             SequenceRecord("same_name1 some_comment", "A", "H"),
             SequenceRecord("same_name2 other_comment", "A", "H"),
@@ -683,23 +689,117 @@ class TestRecordsAreMates:
         )
 
     @pytest.mark.parametrize("number_of_mates", list(range(2, 11)))
-    def test_lots_of_records_are_mates(self, number_of_mates):
+    def test_lots_of_records_are_mates(self, number_of_mates) -> None:
         mates = [SequenceRecord("name", "A", "H") for _ in range(number_of_mates)]
         assert records_are_mates(*mates)
 
-    def test_records_are_not_mates(self):
+    def test_records_are_not_mates(self) -> None:
         assert not records_are_mates(
             SequenceRecord("same_name1 some_comment", "A", "H"),
             SequenceRecord("same_name2 other_comment", "A", "H"),
             SequenceRecord("shame_name3 different_comment", "A", "H"),
         )
 
-    def test_records_are_mates_zero_arguments(self):
+    def test_records_are_mates_zero_arguments(self) -> None:
         with pytest.raises(TypeError) as error:
-            records_are_mates()
+            records_are_mates()  # type: ignore
         error.match("records_are_mates requires at least two arguments")
 
-    def test_records_are_mates_one_argument(self):
+    def test_records_are_mates_one_argument(self) -> None:
         with pytest.raises(TypeError) as error:
-            records_are_mates(SequenceRecord("A", "A", "A"))
+            records_are_mates(SequenceRecord("A", "A", "A"))  # type: ignore
         error.match("records_are_mates requires at least two arguments")
+
+
+class TestBamReader:
+    bam_file = (
+        TEST_DATA / "project.NIST_NIST7035_H7AP8ADXX_TAAGGCGA_1_NA12878"
+        ".bwa.markDuplicates.unmapped.bam"
+    )
+    raw_bam_bytes = gzip.decompress(bam_file.read_bytes())
+    complete_record_with_header = raw_bam_bytes[:6661]
+    complete_header = complete_record_with_header[:6359]
+
+    def test_parse_bam(self):
+        with dnaio.open(self.bam_file) as reader:
+            records = list(reader)
+        assert len(records) == 3
+        assert reader.number_of_records == 3
+        assert records[0].name == "HWI-D00119:50:H7AP8ADXX:1:1104:8519:18990"
+        assert records[0].sequence == (
+            "GATCACAGGTCTATCACCCTATTAACCACTCACGGGAGCTCTCCATGCATTTGGTATTTTCGTCT"
+            "GGGGGGTATGCACGCGATAGCATTGCGAGACGCTGG"
+        )
+        assert records[0].qualities == (
+            "CCCFFFFFHFFHHJIJJIJGGJJJJJJJJJJJJJIGHIIEHIJJJJJJIJJJJIBGGIIIHIIII"
+            "HHHHDD;9CCDEDDDDDDDDDDEDDDDDDDDDDDDD"
+        )
+        assert records[1].name == "HWI-D00119:50:H7AP8ADXX:1:2104:18479:82511"
+        assert records[1].sequence == (
+            "GATCACAGGTCTATCACCCTATTAACCACTCACGGGAGCTCTCCATGCATTTGGTATTTTCGTCT"
+            "GGGGGGTATGCACGCGATAGCATTGCGAGACGCTGG"
+        )
+        assert records[1].qualities == (
+            "CCCFFFFFHFFHHJJJJIJJJJIIJJJJJGIJJJJGIJJJJJJJJGJIJJIJJJGHIJJJJJJJI"
+            "HHHHDD@>CDDEDDDDDDDDDDEDDCDDDDD?BBD9"
+        )
+        assert records[2].name == "HWI-D00119:50:H7AP8ADXX:1:2105:7076:23015"
+        assert records[2].sequence == (
+            "GATCACAGGTCTATCACCCTATTAACCACTCACGGGAGCTCTCCATGCATTTGGTATTTTCGTCT"
+            "GGGGGGTATGCACGCGATAGCATTGCGAGACGCTGG"
+        )
+        assert records[2].qualities == (
+            "@@CFFFDFGFHHHJIIJIJIJJJJJJJIIJJJJIIJIJFIIJJJJIIIGIJJJJDHIJIIJIJJJ"
+            "HHGGCB>BDDDDDDDDDDDBDDEDDDDDDDDDDDDD"
+        )
+
+    def test_parse_header(self):
+        header = (
+            Path(__file__).parent
+            / "data"
+            / "project.NIST_NIST7035_H7AP8ADXX_TAAGGCGA_1_NA12878.bwa"
+            ".markDuplicates.header.sam"
+        )
+        header_bytes = header.read_bytes()
+        with dnaio.open(self.bam_file) as bam:
+            assert bam.header == header_bytes
+
+    @pytest.mark.parametrize(
+        "end", range(len(complete_header) + 1, len(complete_record_with_header))
+    )
+    def test_truncated_record(self, end: int):
+        file = io.BytesIO(self.complete_record_with_header[:end])
+        with pytest.raises(EOFError) as e:
+            list(BamReader(file))
+        e.match("Incomplete record at the end of file")
+
+    @pytest.mark.parametrize("end", [3, 5, 2000, 6000])
+    def test_truncated_header(self, end):
+        file = io.BytesIO(self.complete_record_with_header[:end])
+        with pytest.raises(EOFError) as e:
+            list(BamReader(file))
+        e.match("Truncated BAM file")
+
+    def test_bam_parser_not_binary_error(self):
+        file = io.StringIO(
+            "Don't be too proud of this technological terror you have constructed."
+        )
+        with pytest.raises(TypeError) as error:
+            BamReader(file)
+        error.match("binary IO")
+
+    @pytest.mark.parametrize("buffersize", [4, 8, 10, 20, 40])
+    def test_small_buffersize(self, buffersize):
+        reader = BamReader(str(self.bam_file), buffer_size=buffersize)
+        assert len(list(reader)) == 3
+
+    def test_error_on_mapped_bam(self):
+        bam = TEST_DATA / (
+            "project.NIST_NIST7035_H7AP8ADXX_TAAGGCGA_1_NA12878"
+            ".bwa.markDuplicates.bam"
+        )
+        reader = BamReader(str(bam))
+        it = iter(reader)
+        with pytest.raises(NotImplementedError) as error:
+            next(it)
+        assert error.match("unmapped single reads")
